"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GlobalEventBus = void 0;

var _callback_api = _interopRequireDefault(require("amqplib/callback_api"));

var _v = _interopRequireDefault(require("uuid/v4"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const _process$env = process.env,
      AMQP_USER = _process$env.AMQP_USER,
      AMQP_PASSWORD = _process$env.AMQP_PASSWORD,
      AMQP_HOST = _process$env.AMQP_HOST,
      AMQP_PORT = _process$env.AMQP_PORT,
      AMQP_VHOST = _process$env.AMQP_VHOST;
let vhost = '';

if (AMQP_VHOST) {
  vhost = `/${AMQP_VHOST}`;
}

const AMQP_CONNECT_URL = `amqp://${AMQP_USER}:${AMQP_PASSWORD}@${AMQP_HOST}:${AMQP_PORT}${vhost}`;

class GlobalEventBus {
  /**
   * @private
   */

  /**
   * @private
   */

  /**
   * @private
   * @type {amqp}
   */

  /**
   *
   * @type {string}
   * @private
   */
  constructor({
    amqpConnectUrl = AMQP_CONNECT_URL,
    amqpProvider = _callback_api.default,
    reconnectInterval = 5000
  }) {
    _defineProperty(this, "_connection", null);

    _defineProperty(this, "_channel", null);

    _defineProperty(this, "_amqpProvider", null);

    _defineProperty(this, "_amqpConnectUrl", "");

    _defineProperty(this, "init", async () => {
      console.info(`[AMQP]: Trying create connection`);

      try {
        this._connection = await this._connect();
        this._channel = await this._createChannel();
      } catch (err) {
        throw err;
      }
    });

    _defineProperty(this, "publishEvent", async (type, dataObject) => {
      const jsonObjectBuffer = new Buffer(JSON.stringify(dataObject));
      await this._sendEventWithoutConfirm(type, jsonObjectBuffer);
    });

    _defineProperty(this, "subscribeOnEvent", async (type, handler) => {
      await this._assertQueue(type, {
        durable: false
      });

      this._channel.consume(type, async msg => {
        const data = JSON.parse(msg.content.toString());
        await Promise.resolve(handler(data));

        this._channel.ack(msg);
      });
    });

    this._amqpProvider = amqpProvider;
    this._amqpConnectUrl = amqpConnectUrl;
    this._reconnectInterval = reconnectInterval;
  }
  /**
   * @private
   */


  _createChannel() {
    return new Promise((resolve, reject) => {
      this._connection.createChannel((err, channel) => {
        if (err) {
          return reject(err);
        }

        resolve(channel);
      });
    });
  }
  /**
   * @private
   */


  _connect() {
    return new Promise((resolve, reject) => {
      this._amqpProvider.connect(this._amqpConnectUrl, (err, connection) => {
        if (err) {
          console.error(`[AMQP] Failed connection, reconnecting in ${this._reconnectInterval}: ${err}`);
          setTimeout(this.init, this._reconnectInterval);
        }

        connection.on('error', error => {
          if (error.message !== 'Connection closing') {
            console.error(`[AMQP] connection error: ${error.message}`);
          }
        });
        connection.on('close', () => {
          console.error(`[AMQP] Connection closed, reconnecting in ${this._reconnectInterval}ms`);
          setTimeout(this.init, this._reconnectInterval);
        });
        resolve(connection);
      });
    });
  }
  /**
   * @private
   */


  _assertQueue(type, options) {
    return new Promise((resolve, reject) => {
      this._channel.assertQueue(type, options, (err, q) => {
        if (err) {
          return reject(err);
        }

        resolve(q);
      });
    });
  }
  /**
   * @private
   */


  async _sendEventWithoutConfirm(type, data, options = {}, queueOptions = {}) {
    await this._assertQueue(type, _objectSpread({
      durable: false
    }, queueOptions));

    this._channel.sendToQueue(type, data, _objectSpread({
      persistent: false
    }, options));
  }

}

exports.GlobalEventBus = GlobalEventBus;